@@ -1,5 +1,7 @@
 {-# LANGUAGE CPP #-}
+{-# LANGUAGE ScopedTypeVariables #-}
 {-# LANGUAGE TemplateHaskell #-}
+
 {- |
    Module      : Main
    Copyright   : Copyright (C) 2006-2024 John MacFarlane
@@ -16,24 +18,20 @@ module Main where
 import qualified Control.Exception as E
 import System.Environment (getArgs, getProgName)
 import Text.Pandoc.App ( convertWithOpts, defaultOpts, options
-                       , parseOptionsFromArgs, handleOptInfo )
+                       , parseOptionsFromArgs, handleOptInfo, versionInfo )
 import Text.Pandoc.Error (handleError)
-import System.Exit (exitSuccess)
 import Data.Monoid (Any(..))
-import Control.Monad (when)
 import PandocCLI.Lua
 import PandocCLI.Server
-import qualified Text.Pandoc.UTF8 as UTF8
-import Text.Pandoc.Version (pandocVersion)
-import Text.Pandoc.Data (defaultUserDataDir)
 import Text.Pandoc.Scripting (ScriptingEngine(..))
-import Data.Version (showVersion)
 import qualified Data.Text as T
-
 #ifdef NIGHTLY
 import qualified Language.Haskell.TH as TH
 import Data.Time
 #endif
+#ifdef INCLUDE_WASM
+import PandocWasm()
+#endif
 
 #ifdef NIGHTLY
 versionSuffix :: String
@@ -52,54 +50,41 @@ main = E.handle (handleError . Left) $ do
   let hasVersion = getAny $ foldMap
          (\s -> Any (s == "-v" || s == "--version"))
          (takeWhile (/= "--") rawArgs)
-  when hasVersion versionInfo
+  let versionOr action = if hasVersion then versionInfoCLI else action
   case prg of
-    "pandoc-server.cgi" -> runCGI
-    "pandoc-server"     -> runServer rawArgs
+    "pandoc-server.cgi" -> versionOr runCGI
+    "pandoc-server"     -> versionOr $ runServer rawArgs
     "pandoc-lua"        -> runLuaInterpreter prg rawArgs
     _ ->
       case rawArgs of
         "lua" : args   -> runLuaInterpreter "pandoc lua" args
-        "server": args -> runServer args
-        args           -> do
+        "server": args -> versionOr $ runServer args
+        args           -> versionOr $ do
           engine <- getEngine
           res <- parseOptionsFromArgs options defaultOpts prg args
           case res of
             Left e -> handleOptInfo engine e
             Right opts -> convertWithOpts engine opts
 
-copyrightMessage :: String
-copyrightMessage =
- "Copyright (C) 2006-2024 John MacFarlane. Web: https://pandoc.org\n"
- ++
- "This is free software; see the source for copying conditions. There is no\n"
- ++
- "warranty, not even for merchantability or fitness for a particular purpose."
 
-flagSettings :: String
-flagSettings = "Features: " ++
+getFeatures :: [String]
+getFeatures = [
 #ifdef VERSION_pandoc_server
   "+server"
 #else
   "-server"
 #endif
-  ++ " " ++
+  ,
 #ifdef VERSION_hslua_cli
   "+lua"
 #else
   "-lua"
 #endif
+  ]
 
-versionInfo :: IO ()
-versionInfo = do
-  progname <- getProgName
-  defaultDatadir <- defaultUserDataDir
+versionInfoCLI :: IO ()
+versionInfoCLI = do
   scriptingEngine <- getEngine
-  UTF8.putStr $ T.unlines $ map T.pack
-   [ progname ++ " " ++ showVersion pandocVersion ++ versionSuffix
-   , flagSettings
-   , "Scripting engine: " ++ T.unpack (engineName scriptingEngine)
-   , "User data directory: " ++ defaultDatadir
-   , copyrightMessage
-   ]
-  exitSuccess
+  versionInfo getFeatures
+              (Just $ T.unpack (engineName scriptingEngine))
+              versionSuffix
